#include "stdafx.h"
#include <winsock.h>
#include <atlstr.h>
#include <shlobj.h>
#include "checkinexecution.h"

namespace tipbase
{
	WindowVersion::Type		CheckInExecution::GetWindowVersion()
	{
		WindowVersion::Type nVersion = WindowVersion::win_XP;

		OSVERSIONINFOEX osvi;
		BOOL bOsVersionInfoEx;

		ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
		bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi);

		if( !bOsVersionInfoEx )
		{
			osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
			if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
				return WindowVersion::win_XP;
		}

		switch (osvi.dwPlatformId)
		{
		case VER_PLATFORM_WIN32_NT:
			if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 1)
				nVersion = WindowVersion::win_7;
			else if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 0)
				nVersion = WindowVersion::win_VISTA;		
			else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
				nVersion = WindowVersion::win_2003;
			else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
				nVersion = WindowVersion::win_XP;
			else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
				nVersion = WindowVersion::win_2000;
			else if ( osvi.dwMajorVersion <= 4 )
				nVersion = WindowVersion::win_NT;
			else if ((osvi.dwMajorVersion == 6 && osvi.dwMinorVersion > 1) 
				|| (osvi.dwMajorVersion >= 7))
				nVersion = WindowVersion::win_7_higher;
			break;
		case VER_PLATFORM_WIN32_WINDOWS:
			nVersion = WindowVersion::win_ME_9x;
			break;
		}

		return nVersion;
	}

	BOOL CheckInExecution::CheckMutex(const TCHAR *key)
	{
		if( NULL == CreateMutex( NULL, TRUE, key ) )
		{
			DWORD err = GetLastError();
			return FALSE;
		}

		if (GetLastError() == ERROR_ALREADY_EXISTS)
		{
			return FALSE;
		}

		return TRUE;
	}

	BOOL CheckInExecution::CheckMutex(const TCHAR *key, HANDLE& handle)
	{
		handle = CreateMutex( NULL, TRUE, key );
		if( NULL == handle )
		{
			DWORD err = GetLastError();
			return FALSE;
		}

		if (GetLastError() == ERROR_ALREADY_EXISTS)
		{
			return FALSE;
		}

		return TRUE;
	}

	int	 IsConnected (SOCKET s, fd_set* rd, fd_set* wr, fd_set* ex)
	{
		WSASetLastError( 0 );
		if ( !FD_ISSET(s, rd) && !FD_ISSET(s, wr))
			return 0;
		if ( FD_ISSET(s, ex))
			return 0;
		return 1;
	}

	BOOL CheckConnection(LPCTSTR pszcAddr,int nPort, int nTimeOut)
	{
		int port		= nPort;
		BOOL bConnect	= FALSE;

		WSADATA wsaData;
		int err = WSAStartup( MAKEWORD( 2, 2 ), &wsaData );

		hostent * p = gethostbyname(CT2CA(pszcAddr));
		if(p == NULL)	return FALSE;

		char szIP[16];
		sprintf(szIP, "%d.%d.%d.%d", (BYTE)p->h_addr_list[0][0], (BYTE)p->h_addr_list[0][1], (BYTE)p->h_addr_list[0][2], (BYTE)p->h_addr_list[0][3]);

		SOCKET s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

		if (nTimeOut > 0)
		{
			unsigned long fl = 1;
			if (ioctlsocket(s, FIONBIO, &fl) < 0) // non-blocking setting
			{
				shutdown(s, 1);
				closesocket(s);
				WSACleanup();
				return bConnect;
			}
		}
		SOCKADDR_IN          server_addr;
		server_addr.sin_family = AF_INET;
		server_addr.sin_port = htons(port); 
		server_addr.sin_addr.s_addr =inet_addr(szIP);

		int nResult = connect(s, (SOCKADDR *) &server_addr, sizeof(server_addr)); 
		if (nResult != 0)
		{
			fd_set	rdevents;
			fd_set	wrevents;
			fd_set	exevents;
			struct timeval	tv;

			FD_ZERO(&rdevents);
			FD_SET(s, &rdevents);
			wrevents = rdevents;
			exevents = rdevents;
			tv.tv_sec = nTimeOut;
			tv.tv_usec = 0;
			nResult = select (s+1, &rdevents, &wrevents, &exevents, &tv);

			if (IsConnected(s, &rdevents, &wrevents, &exevents))
			{
				bConnect = TRUE;
			}
		}
		shutdown(s, 1);
		closesocket(s);
		WSACleanup();
		return bConnect;
	}

	BOOL IsAdminRightInVista()
	{
		BOOL retVal = TRUE;
		CRegKey reg;
		LONG lResult = reg.Open(HKEY_LOCAL_MACHINE, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System"),
			STANDARD_RIGHTS_READ | KEY_QUERY_VALUE);
		if (ERROR_SUCCESS == lResult)
		{
			DWORD dwUAC	= -1;
			lResult = reg.QueryDWORDValue(_T("EnableLUA"), dwUAC);

			// PM-696  :   UAC  ¿ ޽  ׷̵ 
			BOOL bIsAdmin = ::IsUserAnAdmin();
			if(bIsAdmin == FALSE && dwUAC == 0)
			{
				retVal = FALSE;
			}
			else
			{
				retVal = TRUE;
			}
		}

		return retVal;
	}

	BOOL IsAdminRightUnderVista()
	{
		HKEY hKey;
		TCHAR NToolsUpdater[] = _T("SOFTWARE\\Ntalkbiz\\NtalkbizUpgrader");
		if (ERROR_SUCCESS != RegCreateKey(HKEY_LOCAL_MACHINE, NToolsUpdater,  &hKey))
			return FALSE;

		if( ERROR_SUCCESS != RegSetValueEx( hKey, _T("CheckAdmin"), 0, REG_SZ, (const BYTE*)"1", 1 ) )
		{
			RegCloseKey(hKey);
			return FALSE;
		}

		RegDeleteValue( hKey, _T("CheckAdmin") );
		RegCloseKey(hKey);
		return TRUE;
	}

	BOOL CheckInExecution::CheckInstallRight()
	{
		if( GetWindowVersion() >= WindowVersion::win_VISTA )
		{
			return IsAdminRightInVista();
		}
		else
		{
			return IsAdminRightUnderVista();
		}
	}

	BOOL CheckInExecution::CheckNetworkStatus(const TCHAR *address, int port, int timeout)
	{
		return CheckConnection(address, port, timeout);
	}
};